
import io
import json
import typing

from . import Document
from beanie.odm.documents import PydanticObjectId

from pydantic import model_validator, ValidationInfo

from .meta import TimestampMixin

from AgentService.utils import string_to_uuid


class Storage(TimestampMixin, Document):
    vector_store_id: str
    key: str

    @classmethod
    async def get_packed(cls, key: str) -> io.BytesIO:
        storage = await Storage.find_one(Storage.key == key)
        storage_items = await StorageItem.find_many(StorageItem.storage_id == storage.id).to_list()

        data = [
            {
                "text": item.data,
                "metadata": {"id": item.key_id}
            }
            for item in storage_items
        ]

        buffer = io.BytesIO()
        buffer.write((json.dumps(data, ensure_ascii=False)).encode("utf-8"))
        buffer.seek(0)

        return buffer


class StorageItem(TimestampMixin, Document):
    storage_id: PydanticObjectId
    data: dict

    key_id: typing.Optional[str] = None

    @model_validator(mode="after")
    def set_unique_id(self, values: ValidationInfo):
        if self.key_id:
            return self

        key = {
            "storage_id": str(self.storage_id),
            "data": self.data.get("key", self.data)
        }
        self.key_id = string_to_uuid(json.dumps(key, sort_keys=True))
        return self




from string import Template
from os import rename
import os
import re


def is_project_name(project_name: str) -> bool:
    return not re.search(r'^[_a-zA-Z]\w*$', project_name) is None


def is_project(project_path: os.PathLike) -> bool:
    return os.path.isfile(os.path.join(project_path, "agent.cfg"))


def process_template(path, **kwargs):
    with open(path, "rb") as f:
        raw = f.read().decode("utf8")

    content = Template(raw).substitute(**kwargs)

    new_path = path[:-len('.tmpl')] if path.endswith(".tmpl") else path

    if path.endswith(".tmpl"):
        rename(path, new_path)

    with open(new_path, "wb") as f:
        f.write(content.encode("utf8"))




from typing import Optional, Dict
from pydantic import BaseModel, Field

from AgentService.enums.response import ResposneStatus


class GetVersionResponse(BaseModel):
    data: Dict = Field(default_factory=dict)
    description: Optional[str] = None
    status: ResposneStatus




from .meta import TimestampMixin
from .tool import Tool
from .message import Message

from ...enums.agent import AgentResponseType


class AgentResponse(TimestampMixin):
    type: AgentResponseType
    content: str | list[Tool] | list[Message] | dict[str, str]




import fastapi
import subprocess

from loguru import logger

from importlib.metadata import version

from .models import (
    GetVersionResponse
)


meta_router = fastapi.APIRouter(prefix="/meta")


@meta_router.get("/version")
async def get_version() -> GetVersionResponse:
    try:
        outer_version = subprocess.run(
            "python3.11 -c \"from importlib.metadata import version; print(version('AgentService'))\"",
            shell=True,
            capture_output=True,
            text=True,
            cwd="."
        ).stdout.replace("\n", "")

    except Exception as err:
        logger.exception(err)

        outer_version = "N/A"

    return GetVersionResponse(
        data={
            "inner": version("AgentService"),
            "outer": outer_version,
        },
        status="ok"
    )
